import "package:collection/collection.dart";
import 'package:flutter_google_places_sdk/flutter_google_places_sdk.dart';
import 'package:verttice_ui/src/base_config.dart';
import 'package:verttice_ui/src/config/config.dart';
import 'package:verttice_ui/src/const/box_sizes.dart';
import "package:verttice_ui/src/const/icons.dart";
import "package:flutter/material.dart";
import "package:flutter_typeahead/flutter_typeahead.dart";
import 'package:verttice_ui/src/widgets/border_label.dart';
import 'package:verttice_ui/src/widgets/dialogs/full_page_dialog.dart';
import 'package:verttice_logic/verttice_logic.dart' as vl;
import 'package:verttice_ui/src/widgets/goolge_maps.dart';
import 'package:verttice_ui/src/widgets/themed_app_bar.dart';
import 'package:verttice_ui/src/widgets/themed_divider.dart';

class GooglePlacesFormFactory extends StatefulWidget {
  final String displayName;
  final String name;
  final Map<String, dynamic>? value;
  final Function(String name, dynamic value) onChanged;
  final FormFieldState<dynamic>? state;
  const GooglePlacesFormFactory({
    Key? key,
    required this.displayName,
    required this.name,
    required this.onChanged,
    this.state,
    this.value,
  }) : super(key: key);

  @override
  _GooglePlacesFormFactoryState createState() =>
      _GooglePlacesFormFactoryState();
}

class _GooglePlacesFormFactoryState extends State<GooglePlacesFormFactory> {
  late Map<String, dynamic>? _value;
  double? _lat;
  double? _lng;

  @override
  void initState() {
    super.initState();
    _value = widget.value;

    _lat = widget.value?["latLng"]?["lat"];
    _lng = widget.value?["latLng"]?["lng"];
  }

  @override
  void didUpdateWidget(GooglePlacesFormFactory oldWidget) {
    super.didUpdateWidget(oldWidget);
    bool eq = const MapEquality().equals(
      _value,
      widget.value,
    );

    if (!eq) {
      _value = widget.value;
      _lat = widget.value?["latLng"]?["lat"];
      _lng = widget.value?["latLng"]?["lng"];
    }
  }

  @override
  Widget build(BuildContext context) {
    return BorderLabel(
      label: widget.displayName,
      child: Tooltip(
        message: _value?["address"] ?? "",
        child: SelectableText(
          _value?["address"] ?? "",
          maxLines: 3,
        ),
      ),
      actions: [
        PhysicalModel(
          color: Colors.grey,
          elevation: 5,
          shape: BoxShape.circle,
          child: CircleAvatar(
            backgroundColor: Colors.red,
            child: InkWell(
              child: ThemedIcons.clear(
                iconStyle: IconStyle(
                  color: Colors.white,
                ),
              ),
              onTap: () {
                widget.onChanged(widget.name, null);
                widget.state?.didChange(null);
              },
            ),
          ),
        ),
        sizedBoxW2,
      ],
      onPressed: () async {
        fullPageDialog(
          context: context,
          builder: (BuildContext context) => SearchPlace(
            lat: _lat,
            lng: _lng,
            placeName: _value?["address"],
            onSelect: (Place data) {
              final addressOnMapData = vl.AddressOnMap(
                textAdress: data.address,
                address: data.addressComponents
                    ?.map((e) => e.name)
                    .toList()
                    .join(", "),
                addressComponents: data.addressComponents
                    ?.map((e) => vl.AddressComponent.fromMap(e.toMap()))
                    .toList(),
                latLng: data.latLng != null
                    ? vl.LatLng.fromMap(data.latLng!.toMap())
                    : null,
                name: data.name,
                types: data.types?.map((e) => e.name).toList(),
                utcOffsetMinutes: data.utcOffsetMinutes,
                viewport: data.viewport != null
                    ? vl.LatLngBounds.fromMap(data.viewport!.toMap())
                    : null,
                websiteUri: data.websiteUri?.toString(),
              ).toMap();

              widget.onChanged(widget.name, addressOnMapData);
              widget.state?.didChange(addressOnMapData);
            },
          ),
        );
      },
    );
  }
}

class SearchPlace extends StatefulWidget {
  final Function(Place value) onSelect;
  final double? lat;
  final double? lng;
  final String? placeName;
  const SearchPlace({
    Key? key,
    required this.onSelect,
    this.lat,
    this.lng,
    this.placeName,
  }) : super(key: key);

  @override
  State<StatefulWidget> createState() => _SearchPlaceState();
}

class _SearchPlaceState extends State<SearchPlace> {
  double? _lat;
  double? _lng;
  String? _placeName;
  Place? _chosenPlace;

  late final FlutterGooglePlacesSdk _places;
  final List<String> _countries = ["uk", "il"];
  final PlaceTypeFilter _placeTypeFilter = PlaceTypeFilter.ADDRESS;
  final List<PlaceField> _placeFields = [
    PlaceField.Address,
    PlaceField.AddressComponents,
    PlaceField.Id,
    PlaceField.Location,
    PlaceField.Name,
    PlaceField.PhoneNumber,
    PlaceField.PlusCode,
    PlaceField.Types,
    PlaceField.UserRatingsTotal,
    PlaceField.UTCOffset,
    PlaceField.Viewport,
    PlaceField.WebsiteUri,
  ];

  @override
  void initState() {
    super.initState();

    _places = FlutterGooglePlacesSdk(VertticeUIConfig.instance.firebaseApiKey);

    _lat = widget.lat;
    _lng = widget.lng;
    _placeName = widget.placeName;
  }

  @override
  void didUpdateWidget(SearchPlace oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_lat != widget.lat || _lng != widget.lng) {
      _lat = widget.lat;
      _lng = widget.lng;
      _placeName = widget.placeName;
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: ThemedAppBar(
        actions: [
          if (_chosenPlace != null)
            Tooltip(
              message: "Select place",
              child: IconButton(
                onPressed: () {
                  widget.onSelect(_chosenPlace ?? const Place());

                  Navigator.of(context).pop();
                },
                icon: ThemedIcons.check(),
              ),
            ),
        ],
        title: "Find a place",
      ),
      body: Padding(
        padding: const EdgeInsets.all(30),
        child: LayoutBuilder(
          builder: (context, constraints) {
            return Flex(
              direction: constraints.maxWidth < mobileWidth
                  ? Axis.vertical
                  : Axis.horizontal,
              crossAxisAlignment: CrossAxisAlignment.start,
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: [
                SizedBox(
                  width: (constraints.maxWidth / 2) - 10,
                  child: ListView(
                    controller: ScrollController(),
                    children: _buildPredictionWidgets(),
                  ),
                ),
                ThemedDivider(width: 2, height: constraints.maxHeight),
                SizedBox(
                  width: (constraints.maxWidth / 2) - 10,
                  child: ListView(
                    controller: ScrollController(),
                    children: [
                      SizedBox(
                        height: constraints.maxHeight == double.infinity
                            ? 600
                            : constraints.maxHeight,
                        width: (constraints.maxWidth / 2) - 10,
                        child: MapCoordinatesPage(
                          lat: _lat ?? 54.9506422,
                          lng: _lng ?? -1.603025,
                          placeName: _placeName ?? "CP12 Energy Headquaters",
                        ),
                      )
                    ],
                  ),
                ),
              ],
            );
          },
        ),
      ),
    );
  }

  List<Widget> _buildPredictionWidgets() {
    return [
      TypeAheadField(
        textFieldConfiguration: TextFieldConfiguration(
          autofocus: true,
          style: DefaultTextStyle.of(context)
              .style
              .copyWith(fontStyle: FontStyle.italic),
          decoration: const InputDecoration(
            border: OutlineInputBorder(),
            hintText: "Search a place",
          ),
        ),
        suggestionsCallback: (pattern) async {
          List<AutocompletePrediction> predictions = [];
          if (pattern == "") {
            return predictions;
          }
          final result = await _places.findAutocompletePredictions(
            pattern,
            countries: _countries,
            placeTypeFilter: _placeTypeFilter,
            newSessionToken: false,
          );
          return result.predictions;
        },
        itemBuilder: (context, AutocompletePrediction suggestion) {
          return ListTile(
            leading: ThemedIcons.pin(),
            title: Text(suggestion.fullText),
            subtitle: Text(
              suggestion.primaryText + " - " + suggestion.secondaryText,
            ),
          );
        },
        onSuggestionSelected: (AutocompletePrediction suggestion) async {
          final result = await _places.fetchPlace(suggestion.placeId,
              fields: _placeFields);
          setState(() {
            _lat = result.place?.latLng?.lat;
            _lng = result.place?.latLng?.lng;
            _placeName = suggestion.fullText;
            _chosenPlace = result.place;
          });
        },
      ),
    ];
  }
}
